#include <Wire.h>
#include "Adafruit_MPRLS.h"
#include "Adafruit_MPR121.h"

// ブザー出力する端子
#define BUZZER_PIN 25
// ブザーを鳴らすときの圧力差　
#define PRS_OFFSET 10.0

//音階名と周波数の対応
#define C4 261.6
#define Cs4 277.18
#define D4 293.665
#define Ds4 311.127
#define E4 329.63
#define F4 349.228
#define Fs4 369.994
#define G4 391.995
#define Gs4 415.305
#define A4 440
#define As4 466.164
#define B4 493.883
#define C5 523.251

// You can have up to 4 on one i2c bus but one is enough for testing!
Adafruit_MPR121 cap = Adafruit_MPR121();

// You dont *need* a reset and EOC pin for most uses, so we set to -1 and don't connect
#define RESET_PIN  -1  // set to any GPIO pin # to hard-reset on begin()
#define EOC_PIN    -1  // set to any GPIO pin to read end-of-conversion by pin
Adafruit_MPRLS mpr = Adafruit_MPRLS(RESET_PIN, EOC_PIN);

//プログラムで使用するグローバル変数の定義 
char texts[50];   // 文字列出力用
float PrsInit;    // 圧力センサ初期値
int LastOn = -1;  // 最後に出した音の記憶用(-1は無音)

void setup() {
  Serial.begin(115200);
  ledcAttachPin(BUZZER_PIN,1);

  Serial.println("Wind Synthesizer");

  // タッチセンサをアドレス0x05Aで初期化する
  if (!cap.begin(0x5A)) {
    Serial.println("MPR121 not found, check wiring?");
    while (1);
  }
  Serial.println("MPR121 found!");

  // 圧力センサを初期化する（デフォルトのアドレス0x18）
  if (! mpr.begin()) {
    Serial.println("Failed to communicate with MPRLS sensor, check wiring?");
    while (1) {
      delay(10);
    }
  }
  Serial.println("Found MPRLS sensor");

  // 何もない状態の圧力センサの値を取得する
  float prsTotal = 0;
  for (uint8_t ii = 0 ; ii < 10 ; ii++){
    prsTotal += mpr.readPressure();   // 圧力センサから圧力取得
    delay(10);
  }
  PrsInit = prsTotal / 10;
  sprintf(texts, "Prs Init:%f\n", PrsInit);
  Serial.print(texts);
}

void loop() {
  float nowPrs = mpr.readPressure();  // 圧力センサから圧力取得
  uint16_t nowTouch = cap.touched();  // タッチセンサの状態取得

  // タッチセンサの値を0と1並びの文字に変換する
  char bitptn[13];
  int firstOn = -1;
  for (uint8_t i=0; i<12; i++) {
    if ((nowTouch & ( 1<<i))==0){
      bitptn[i] = '0';
    }else{
      if (firstOn < 0) firstOn = i;
      bitptn[i] = '1';
    }
  }
  bitptn[12] = '\0';

  if (nowPrs >= (PrsInit+PRS_OFFSET)){
    // 圧力が高いときは音を出す
    if (firstOn != LastOn) {
      ringBuzzer(firstOn, 1);
      LastOn = firstOn;
    }
  }else{
    // 音を止める
    ringBuzzer(-1, 1);
    LastOn = -1;
  }

  sprintf(texts, "Prs:%f Tch:%s\n", nowPrs, bitptn);
  Serial.print(texts);
  delay(50);
}

void ringBuzzer(int argBit, float argRate){
  float freq;
  switch(argBit){
    case 0: freq = C4; break;
    case 1: freq = D4; break;
    case 2: freq = E4; break;
    case 3: freq = F4; break;
    case 4: freq = G4; break;
    case 5: freq = A4; break;
    case 6: freq = B4; break;
    case 7: freq = C5; break;
    default: freq = 0; break;
  }
  ledcWriteTone(1,freq*argRate);
}
